<?php
namespace Ep\Gath\WordPress;

use Exception;
use WC_Comments;
use WC_Product;
use WC_Product_Data_Store_CPT;

class WooDataStore extends WC_Product_Data_Store_CPT
{
	/** @var ProductProvider */
	protected $products;
	/** @var PriceProvider */
	protected $prices;
	/** @var StockProvider */
	protected $stocks;

	/**
	 * @return ProductProvider
	 */
	protected function getProductProvider()
	{
		if (null === $this->products) {
			$services       = ep_services();
			$this->products = $services->get('GathProductProvider');
		}

		return $this->products;
	}

	/**
	 * @return PriceProvider|mixed
	 */
	protected function getPriceProvider()
	{
		if (null === $this->prices) {
			$services = ep_services();
			$this->prices = $services->get('GathPriceProvider');
		}

		return $this->prices;
	}

	/**
	 * @return StockProvider|mixed
	 */
	protected function getStockProvider()
	{
		if (null === $this->stocks) {
			$services = ep_services();
			$this->stocks = $services->get('GathStockProvider');
		}

		return $this->stocks;
	}

	/**
	 * Method to read a product from the database.
	 *
	 * @param WC_Product $product
	 * @throws Exception
	 */
	public function read(&$product) {
		$product->set_defaults();

		if ( ! $product->get_id() || ! ( $post_object = $this->getProductProvider()->getProduct($product->get_id() ) ) || 'product' !== $post_object->post_type ) {
			throw new Exception( __( 'Invalid product.', 'woocommerce' ) );
		}

		$product->set_props(array(
			'name'              => $post_object->post_title,
			'slug'              => $post_object->post_name,
			'date_created'      => 0 < $post_object->post_date_gmt ? wc_string_to_timestamp( $post_object->post_date_gmt ) : null,
			'date_modified'     => 0 < $post_object->post_modified_gmt ? wc_string_to_timestamp( $post_object->post_modified_gmt ) : null,
			'status'            => $post_object->post_status,
			'description'       => $post_object->post_content,
			'short_description' => $post_object->post_excerpt,
			'parent_id'         => $post_object->post_parent,
			'menu_order'        => $post_object->menu_order,
			'reviews_allowed'   => 'open' === $post_object->comment_status,
		));

		$this->read_attributes( $product );
		$this->read_downloads( $product );
		$this->read_visibility( $product );
		$this->read_product_data( $product );
		$this->read_extra_data( $product );
		$product->set_object_read( true );
	}

	/**
	 * Read product data. Can be overridden by child classes to load other props.
	 *
	 * @param WC_Product $product
	 * @since 3.0.0
	 */
	protected function read_product_data( &$product ) {
		$id = $product->get_id();

		if ( '' === ( $review_count = get_post_meta( $id, '_wc_review_count', true ) ) ) {
			WC_Comments::get_review_count_for_product( $product );
		} else {
			$product->set_review_count( $review_count );
		}

		if ( '' === ( $rating_counts = get_post_meta( $id, '_wc_rating_count', true ) ) ) {
			WC_Comments::get_rating_counts_for_product( $product );
		} else {
			$product->set_rating_counts( $rating_counts );
		}

		if ( '' === ( $average_rating = get_post_meta( $id, '_wc_average_rating', true ) ) ) {
			WC_Comments::get_average_rating_for_product( $product );
		} else {
			$product->set_average_rating( $average_rating );
		}

		$products = $this->getProductProvider();
		$prices = $this->getPriceProvider();
		$stocks = $this->getStockProvider();

		$products->getProduct($id);
		$gathProduct = $products->getGathProduct($id);

		$product->set_props( array(
			'sku'                => $gathProduct->getSku(),
			'regular_price'      => $prices->getMetadata($id, '_regular_price'),
			'sale_price'         => $prices->getMetadata($id, '_sale_price'),
			'price'              => $prices->getMetadata($id, '_price'),
			'date_on_sale_from'  => '',
			'date_on_sale_to'    => '',
			'total_sales'        => get_post_meta( $id, 'total_sales', true ),
			'tax_status'         => 'taxable',
			'tax_class'          => $gathProduct->getTaxClass(),
			'manage_stock'       => $stocks->getMetadata($id, '_manage_stock'),
			'stock_quantity'     => $stocks->getMetadata($id, '_stock'),
			'stock_status'       => $stocks->getMetadata($id, '_stock_status'),
			'backorders'         => $stocks->getMetadata($id, '_backorders'),
			'sold_individually'  => get_post_meta( $id, '_sold_individually', true ),
			'weight'             => 0,
			'length'             => get_post_meta( $id, '_length', true ),
			'width'              => get_post_meta( $id, '_width', true ),
			'height'             => get_post_meta( $id, '_height', true ),
			'upsell_ids'         => get_post_meta( $id, '_upsell_ids', true ),
			'cross_sell_ids'     => get_post_meta( $id, '_crosssell_ids', true ),
			'purchase_note'      => get_post_meta( $id, '_purchase_note', true ),
			'default_attributes' => get_post_meta( $id, '_default_attributes', true ),
			'category_ids'       => [],
			'tag_ids'            => [],
			'shipping_class_id'  => $gathProduct->getShippingClassId(),
			'virtual'            => get_post_meta( $id, '_virtual', true ),
			'downloadable'       => get_post_meta( $id, '_downloadable', true ),
			'gallery_image_ids'  => array_filter(explode(',', get_post_meta($id, '_product_image_gallery', true))),
			'download_limit'     => get_post_meta( $id, '_download_limit', true ),
			'download_expiry'    => get_post_meta( $id, '_download_expiry', true ),
			'image_id'           => get_post_meta($id,'_thumbnail_id', true),
            'featured' => $gathProduct->isFeatured(),
		));

		// Handle sale dates on the fly in case of missed cron schedule.
		if ( $product->is_type( 'simple' ) && $product->is_on_sale( 'edit' ) && $product->get_sale_price( 'edit' ) !== $product->get_price( 'edit' ) ) {
			update_post_meta( $product->get_id(), '_price', $product->get_sale_price( 'edit' ) );
			$product->set_price( $product->get_sale_price( 'edit' ) );
		}
	}
}