<?php
namespace Ep\WooCommerce;

use Ep\Gath\GathService;
use Ep\Gath\WordPress\ProductProvider;
use Ep\WooCommerce\Entity\ProductOrderItem;
use Ep\WordPress\Exception\RuntimeException;
use Ep\WordPress\Service\AbstractWordPressService;
use Psr\Container\ContainerInterface;
use WC_Order;
use WC_Order_Item;
use WC_Product;

/**
 * Class OrderService
 *
 * @package Ep\WooCommerce
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class OrderService extends AbstractWordPressService
{
    /** @var ProductProvider */
	protected $products;

	/**
	 * @return ProductProvider|mixed
	 */
	protected function getProductProvider()
    {
        if (null === $this->products) {
            $this->products = $this->services->get('GathProductProvider');
        }

        return $this->products;
    }

	/**
	 * @param $productId
	 *
	 * @return null|\WP_Post
	 */
    protected function getProduct($productId)
    {
        $provider = $this->getProductProvider();
        return $provider->getProduct($productId);
    }

	/**
	 * Kačimo se na WordPRess
	 */
	public function attach(): void
	{
		add_action('woocommerce_checkout_order_processed', [$this, 'onOrderSave'], 10, 3);
		add_filter('woocommerce_checkout_create_order_line_item_object', [$this, 'createOrderLineItem']);
		add_filter('woocommerce_get_order_item_classname', function($classname, $itemType) {
		    if (in_array($itemType, ['product', 'line_item'])) {
		        return ProductOrderItem::class;
            }

		    return $classname;
        }, 10, 2);
		add_action('woocommerce_checkout_create_order_line_item', [$this, 'adjustOrderLineItem'], 10, 3);

		add_filter('wc_round_tax_total', function($rounded, $value) {
		    return floatval($value);
        }, 10, 2);
	}

	/**
	 * Akcije za admin.
	 */
	public function attachAdmin(): void
	{
		add_action('woocommerce_admin_order_item_headers', [$this, 'orderItemHeaders']);
		add_action('woocommerce_admin_order_item_values', [$this, 'orderItemValues'], 10, 3);
		add_filter('woocommerce_hidden_order_itemmeta', [$this, 'hiddenOrderMeta']);
	}

	/**
	 * @param WC_Order_Item $item
	 *
	 * @return ProductOrderItem|WC_Order_Item
	 */
	public function createOrderLineItem(WC_Order_Item $item)
    {
        if ($item instanceof \WC_Order_Item_Product) {
            return new ProductOrderItem();
        }

        return $item;
    }

    public function adjustOrderLineItem(WC_Order_Item $item, $cartItemKey, $values)
    {
        if ($item instanceof \WC_Order_Item_Product) {
            $item->set_product_id(absint($values['product_id']));
        }
    }

	/**
	 * Zaglavlja tablice sa stavkama narudžbe.
	 */
	public function orderItemHeaders()
	{
		?>
		<th class="regular_price">Cijena bez popusta/rabata</th>
		<th class="discount">Popust/rabat</th>
		<?php
	}

	/**
	 * Ispisuje dodatne vrijednosti stavke u narudžbi.
	 *
	 * @param $product
	 * @param $item
	 * @param $itemId
	 */
	public function orderItemValues($product, $item, $itemId)
	{
		?>
		<td><?= wc_price(wc_get_order_item_meta($itemId, '_discount_regular_price', true)); ?></td>
		<td><?= intval(wc_get_order_item_meta($itemId, '_discount_percentage', true)); ?>%</td>
		<?php
	}

	/**
	 * Sakriva određene meta podatke sa stranice narudžbe.
	 *
	 * @param array $metaKeys
	 *
	 * @return array
	 */
	public function hiddenOrderMeta(array $metaKeys)
	{
		$metaKeys[] = '_discount_percentage';
		$metaKeys[] = '_discount_regular_price';

		return $metaKeys;
	}

	/**
	 * @param int $orderId
	 * @param $postedData
	 * @param WC_Order $order
	 */
	public function onOrderSave(int $orderId, $postedData, WC_Order $order)
	{
		$config = get_option('ep_webshop', []);
		update_post_meta($orderId, '_order_type', $config['type'] ?: 'retail');

		foreach ($order->get_items() as $item) {
			/** @var WC_Order_Item $item */
			$this->saveOrderItemMeta($item->get_id(), $item);
		}
	}

	/**
	 * @param $itemId
	 * @param $cartItem
	 */
	public function saveOrderItemMeta($itemId, $cartItem)
	{
		$config = get_option('ep_webshop', []);
		$type = $config['type'] ?? 'retail';

		$gathConfig = $this->getConfig('gath');

		$productId = absint(wc_get_order_item_meta($itemId, '_variation_id', true));

		if (0 === $productId) {
			$productId = absint(wc_get_order_item_meta($itemId, '_product_id', true));
		}

		if ('1' === $gathConfig['enabled'] && in_array('prices', $gathConfig['modules'])) {
			switch ($type) {
				case 'retail':
					$discountPercentage = get_post_meta($productId, '_sale_percentage', true);
					$discountRegularPrice = get_post_meta($productId, '_regular_price_no_tax', true);
					break;
				case 'wholesale':
					$discountPercentage = get_post_meta($productId, '_rebate_percentage', true);
					$discountRegularPrice = get_post_meta($productId, '_net_price', true);
					break;
				default:
					throw new RuntimeException(sprintf("Nepoznata vrsta Webshopa '%s'", $type));
			}
        } else {
		    if ('wholesale' === $type) {
		        throw new \RuntimeException('Ne možete imati veleprodajni shop bez servisa!');
            }

			$discountRegularPrice = get_post_meta($productId, '_regular_price', true);
            $salePrice = get_post_meta($productId, '_sale_price', true);

            if ($discountRegularPrice !== $salePrice) {
	            $product = wc_get_product($productId);

	            if (wc_prices_include_tax()) {
		            $discountRegularPrice = wc_get_price_excluding_tax($product, [
			            'price' => $discountRegularPrice,
		            ]);

		            $salePrice = wc_get_price_excluding_tax($product, [
                        'price' => $salePrice,
                    ]);
                }
            }

            $discountPercentage = round(($discountRegularPrice - $salePrice) / $discountRegularPrice * 100, 2);
        }

		wc_update_order_item_meta($itemId, '_discount_percentage', floatval($discountPercentage));
		wc_update_order_item_meta($itemId, '_discount_regular_price', floatval($discountRegularPrice));
	}
}